@@ -13,6 +13,8 @@
 # OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 
 import logging
+logger = logging.getLogger(__name__)
+
 import os
 
 import networkx as nx
@@ -174,7 +176,11 @@ class GraphomalyEstimator(BaseEstimator):
         self.best_params = {}
         self.best_score_ = {}
 
-        logging.basicConfig(level=logging.INFO)
+        # For each model, a list of generators used to generate parameter combinations
+        # alongside the parameters defined by `models_fit_kwargs`
+        self.params_generators = {}
+
+        logger.setLevel(logging.INFO)
 
         # Overwrite from configuration file
         if self.config_file and os.path.exists(self.config_file):
@@ -348,7 +354,7 @@ class GraphomalyEstimator(BaseEstimator):
         else:
             self.models_list = self.models_subset
 
-    def fit(self, X, y=None):
+    def fit(self, X, y=None, refit=False):
         """Perform anomaly detection on samples in `X`.
 
         Parameters
@@ -360,6 +366,9 @@ class GraphomalyEstimator(BaseEstimator):
             The ground truth for samples `X`. If `None` then the best estimator
             across parameter opotions will not be sought.
 
+        refit: boolean
+            Reuse saved experiments (refit=False) or run them again (refit=True). Defaults to False.
+
         Returns
         -------
         self: object
@@ -371,10 +380,10 @@ class GraphomalyEstimator(BaseEstimator):
             example of performing fit on synthetic
             generated data on a small subset of methods and parameters list.
         """
-        if self._model_is_fitted is True:
+        if refit is False and self._model_is_fitted is True:
             return self
 
-        self.tune(X, y)
+        self.tune(X, y, refit=refit)
         self._model_is_fitted = True
 
         return self
@@ -494,6 +503,11 @@ class GraphomalyEstimator(BaseEstimator):
         """
         for algorithm in self.models_list:
             model_kwargs, fit_kwargs = self._get_params(algorithm, X.shape[1])
+
+            # Merge 'common' kwargs, with lower priority
+            if 'common' in self.models_fit_kwargs:
+                fit_kwargs = {**self.models_fit_kwargs['common'], **fit_kwargs}
+
             clf = ModelsLoader.get(algorithm, **model_kwargs)
             if hasattr(clf, "save"):  # tf model detected
                 clf_type = "tensorflow"
@@ -507,6 +521,9 @@ class GraphomalyEstimator(BaseEstimator):
                 datadir=self.results_path,
                 clf_type=clf_type,
                 refit=refit,
+                clf_algorithm = algorithm,
+                clf_model_kwargs = model_kwargs,
+                params_generators = self.params_generators[algorithm] if algorithm in self.params_generators else []
             )
             search.fit(X, y)
 
@@ -523,7 +540,7 @@ class GraphomalyEstimator(BaseEstimator):
 
                 self.models.append(search.best_estimator_)
 
-                logging.info(
+                logger.info(
                     f"Best params for {algorithm}[{search.best_score_}]: "
                     f"{search.best_params_}"
                 )
