@@ -12,11 +12,18 @@
 # ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
 # OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 
-import hashlib
 import logging
+logger = logging.getLogger(__name__)
+
+import hashlib
+import itertools
+import logging
+import math
 import multiprocessing
 import os
 import pickle
+import pprint
+import shutil
 import time
 from collections.abc import Iterable
 from pathlib import Path
@@ -27,6 +34,7 @@ from sklearn.metrics import confusion_matrix
 from sklearn.model_selection import ParameterGrid
 from tensorflow.keras.models import load_model
 
+from .models import ModelsLoader
 
 class GridSearch:
     """Grid Search routine.
@@ -201,13 +209,20 @@ class GridSearch:
         get_results=None,
         compare_metric=None,
         refit=False,
+        clf_algorithm = '',
+        clf_model_kwargs={},
+        params_generators = []
+
     ):
         self.base_clf = clf
+        self.clf_algorithm = clf_algorithm
+        self.clf_model_kwargs = clf_model_kwargs
         self.datadir = datadir
         self.n_cpus = n_cpus
         self.clf_type = clf_type
 
-        self.params = params
+        self.params = params        # Dictionary with params settings
+        self.params_grid = []       # Will hold the actual parameter configurations returned by ParameterGrid
         self.refit = refit
 
         self.clf_name = clf.__class__.__name__
@@ -228,6 +243,9 @@ class GridSearch:
         self.labels_ = None
         self.estimators_ = []
 
+        # A list of generators used to generate parameter combinations alongside the ones defined by `models_fit_kwargs`
+        self.params_generators = params_generators
+
         logging.basicConfig(level=logging.INFO)
 
     def _compare_results(self, experiment, clf, metrics):
@@ -287,6 +305,85 @@ class GridSearch:
             )
         return metrics
 
+    @staticmethod
+    def _test_clf_static(params):
+        """Static version of _test_clf(). Instantiates tensorflow models internally.
+
+        Args:
+            params (dict): Dictionary with all necessary parameters. Includes parameters for instantiating a tensorflow model.
+
+        Returns:
+            dict: Metrics dict
+        """
+        X = params.pop("samples")
+        y = params.pop("targets")
+        clf_algorithm = params.pop("clf_algorithm")
+        clf_model_kwargs = params.pop("clf_model_kwargs")
+        clf_type = params.pop("clf_type")
+        datadir = params.pop("datadir")
+        clf_name = params.pop("clf_name")
+        experiment_file_params = params.pop("experiment_file_params")
+        start = time.time()
+
+        logger.info(f"Running {experiment_file_params}")
+
+        # Assume self.clf_type == "tensorflow"
+        clf = ModelsLoader.get(clf_algorithm, **clf_model_kwargs)
+
+        clf = clf.set_params(**params)
+        y_pred = clf.fit_predict(X)
+
+        metrics = _fpfn(None, None, y_pred, y)
+        end = time.time()
+        duration = end - start
+        metrics["duration"] = duration
+
+        logger.debug(metrics)
+
+        fname = f"{datadir}/{clf_name}" + experiment_file_params
+
+        if clf_type == "tensorflow":  # tf is not pickle compatible
+
+            # Save model
+            clf.metrics_ = metrics
+            clf.save(fname)
+
+            # Manually pickle attributes in a separate file
+            attrs_to_pickle = clf.get_config()
+            attrs_to_pickle['optimizer'] = params['optimizer']
+            attrs_to_pickle['loss'] = params['loss']
+            pickle.dump(attrs_to_pickle, open(os.path.join(fname, 'autoencoder_attrs'),'wb'))
+
+            # Pickle metrics
+            pickle.dump(
+                # [ba, tpr, tnr, duration, tn, fp, fn, tp],
+                metrics,
+                open(
+                    fname + "/metrics",
+                    "wb",
+                ),
+            )
+
+            # Save metrics also as a text file
+            with open(os.path.join(fname, 'metrics.txt'), "w") as external_file:
+                pprint.pprint(metrics, stream=external_file)
+
+            # Save config to a text file
+            with open(os.path.join(fname, 'config.txt'), "w") as external_file:
+                pprint.pprint(clf.get_config(), stream=external_file)
+
+        else:
+            metrics["clf"] = clf
+            pickle.dump(
+                # [clf, ba, tpr, tnr, duration, tn, fp, fn, tp],
+                metrics,
+                open(
+                    fname,
+                    "wb",
+                ),
+            )
+        return metrics
+
     def _grid_make_iterable(self):
         for k, v in self.params.items():
             if not isinstance(self.params[k], Iterable):
@@ -299,31 +396,48 @@ class GridSearch:
         self.params["targets"] = [y]
         self._grid_make_iterable()
 
-        self.params = list(ParameterGrid(self.params))
+        # Make a parameter grid based on the `self.params` values
+        self.params_grid = list(ParameterGrid(self.params))
+
+        # Function to merge tuple of dicts
+        def _merge_dict_tuple(tup):
+            """ Function to merge a tuple of dictionaries into a single output dictionary
+            """
+            d = {}
+            for t in tup:
+                d.update(t)
+            return d
+
+        # Combine `self.params` with the parameters defined by the additional generators
+        # Generators can overwrite values defined in self.params
+        self.params_grid = [_merge_dict_tuple(tup) for tup in itertools.product(self.params_grid, *self.params_generators)]
 
     def _experiment_get_path(self, prefix, p):
         return prefix + self._experiment_file_params(p)
 
     def _experiment_exists(self, prefix, p):
         experiment = self._experiment_get_path(prefix, p)
-        return os.path.isfile(experiment)
+        return os.path.exists(experiment)
 
     def _experiment_remove_if_exists(self, prefix, p):
         experiment = self._experiment_get_path(prefix, p)
-        if os.path.isfile(experiment):
-            os.remove(experiment)
+        if os.path.exists(experiment):
+            shutil.rmtree(experiment)            
 
     def _experiment_file_params(self, p):
         str_params = ""
         for k, v in p.items():
-            if k == "samples" or k == "targets":
+            if k == "samples" or k == "targets" or k == "savename_key_len":
                 continue
             if type(v) == np.ndarray:
                 v = self._params_array_subsample_hash(v)
-            str_params += f"-{k}_{v}"
+            keylen = p['savename_key_len'] if 'savename_key_len' in p else math.inf
+            str_params += f"-{k[:min(keylen,len(k))]}_{v}"    # Restrict key length to first keylen characters
+        str_params = str_params.replace('[','_').replace(']','_').replace(',','_') 
         return str_params
 
-    def _params_array_subsample_hash(self, arr, n_digits=8):
+    @staticmethod
+    def _params_array_subsample_hash(arr, n_digits=8):
         # inds = np.random.randint(low=0, high=arr.size, size=np.minimum(1000, arr.size))
         # sampled_arr = arr.flat[inds]
         step = np.maximum(1, round(0.1 * arr.size))
@@ -338,14 +452,14 @@ class GridSearch:
 
     def _grid_remove_existing_experiments(self):
         prefix = f"{self.datadir}/{self.clf_name}"
-        self.params_in_ = self.params
-        for p in self.params:
+        self.params_in_ = self.params_grid
+        for p in self.params_grid:
             self._experiment_remove_if_exists(prefix, p)
 
     def _grid_remove_existing_tests(self):
         prefix = f"{self.datadir}/{self.clf_name}"
-        self.params_in_ = self.params
-        self.params = [p for p in self.params if not self._experiment_exists(prefix, p)]
+        self.params_in_ = self.params_grid
+        self.params_grid = [p for p in self.params_grid if not self._experiment_exists(prefix, p)]
 
     def _grid_find_best_result(self):
         # p = Path(self.datadir)
@@ -358,6 +472,15 @@ class GridSearch:
             logging.info(f"{experiment}")
             if os.path.isdir(experiment):  # tf
                 clf = load_model(experiment)
+
+                # Restore model, including the manually saved attributes
+                with open(os.path.join(str(experiment), "autoencoder_attrs"), "rb") as fp:
+                    clf_attrs = pickle.load(fp)
+                # Change object's class to actual Autoencoder, so methods are available. Weird but works.
+                clf.__class__ = ModelsLoader.get_constructor(self.clf_algorithm)
+                # Update with the manually pickled attributes
+                clf.set_params(**clf_attrs)
+
                 with open(str(experiment) + "/metrics", "rb") as fp:
                     metrics = pickle.load(fp)
             else:
@@ -415,13 +538,33 @@ class GridSearch:
         else:
             self._grid_remove_existing_tests()
 
-        # tf incompatible with mp according to tf issue #46316
-        if self.n_cpus == 1 or self.clf_type == "tensorflow":
-            for p in self.params:
+        if self.n_cpus == 1:
+            for p in self.params_grid:
                 self._test_clf(p)
+
+        # tf incompatible with mp according to tf issue #46316
+        elif self.clf_type == "tensorflow":
+
+            # Append new parameters so that _test_clf_static() can instantiate the model internally
+            new_params = {  "clf_algorithm": self.clf_algorithm,
+                            "clf_model_kwargs": self.clf_model_kwargs,
+                            "clf_type": self.clf_type,
+                            "datadir": self.datadir,
+                            "clf_name": self.clf_name,
+            }
+            for p in self.params_grid:
+                p.update({"experiment_file_params": self._experiment_file_params(p)})
+                p.update(new_params)
+
+                self._test_clf_static(p)    
+
+            # Still can't parallelize, models get instantiated but hang when fitting()
+            # pool_obj = multiprocessing.Pool(self.n_cpus)
+            # pool_obj.map(self._test_clf_static, self.params_grid)
+
         else:
             pool_obj = multiprocessing.Pool(self.n_cpus)
-            pool_obj.map(self._test_clf, self.params)
+            pool_obj.map(self._test_clf, self.params_grid)
 
     def fit(self, X, y=None):
         """Perform grid search on samples in `X` with the selected `clf`.
