@@ -7,6 +7,7 @@ import pandas as pd
 
 # ---------------------------------------------------------------------------------
 
+
 def graph_to_egonet_features(
     G,
     FULL_egonets=True,
@@ -21,10 +22,10 @@ def graph_to_egonet_features(
     ----------
     G : networkx DiGraph or pandas DataFrame or matrix
         Transactions graph. It must have the following edge attributes:
-        
+
         * `'cumulated_amount'`: the cumulated amount of transactions from source to destination node
         * `'n_transactions'`: number of transactions between the two nodes
-        
+
         It can have other attributes.
         A dataframe is immediately converted to Networkx graph, so there is no advantage in giving a dataframe.
         The columns for nodes forming an edge should be named `'id_source'` and `'id_destination'`.
@@ -57,7 +58,7 @@ def graph_to_egonet_features(
     -------
     node_features : pandas dataframe (if input was graph or dataframe) or matrix (if input was matrix)
         Contains the following columns (in this order):
-        
+
         * `'f_degree_in'`: in degree of the node
         * `'f_degree_out'`: out degree of the node
         * `'f_amount_in'`: total amount received by node from neighbors
@@ -87,10 +88,10 @@ def graph_to_egonet_features(
     # BD 19.08.2021
     # BD 03.02.2022 revised to accept matrix input-output
     # BD 16.03.2022 labels may be given and transformed
-    
+
     matrix_io = False
     Ns = len(summable_attributes)
-    
+
     # add labels as edge attributes directly to graph, if graph input
     if isinstance(G, nx.DiGraph):
         if labels is not None:
@@ -212,7 +213,7 @@ def graph_to_egonet_features(
         node_labels = np.zeros(Nn)
     else:
         node_labels = None
-    
+
     # go over all nodes and extract features
     row = 0
     for node in G:
@@ -269,11 +270,11 @@ def graph_to_egonet_features(
 
         # Same operations on reduced egonets, which are obtained by removing lonely nodes from egonet
         # (a lonely node is connected only with a single edge to the central node of the egonet)
-        enodes = np.zeros(len(Gs))
+        enodes = []
         i_ego = 0
         for nn in Gs:
             if Gs.degree(nn) > 1:  # keep only nodes that have more than one neighbor
-                enodes[i_ego] = nn
+                enodes.append(nn)
                 i_ego += 1
         # if something left in the egonet (i.e., not a star)
         if i_ego > 0:
@@ -346,7 +347,7 @@ def graph_to_egonet_features(
             "f_egored_nr_edges",
         ] + summable_attributes
         node_features = pd.DataFrame(node_features, columns=df_columns)
-        
+
     return node_features, node_ids, node_labels
 
 
@@ -467,13 +468,9 @@ def compute_derivated_egonet_feature(
     # BD 9.02.2022
 
     action_dict = {
-        "f_average_amount_in": [
-            "div", 
-            2, 
-            4
-        ],  # "f_amount_in" / "f_nr_trans_in"
+        "f_average_amount_in": ["div", 2, 4],  # "f_amount_in" / "f_nr_trans_in"
         "f_average_amount_out": [
-            "div", 
+            "div",
             3,
             5,
         ],  # "f_amount_out" / "f_nr_trans_out"
@@ -487,11 +484,7 @@ def compute_derivated_egonet_feature(
             11,
             13,
         ],  # "f_egored_amount_out" / "f_egored_nr_trans_out"
-        "f_egored_degree_in_rel": [
-            "div", 
-            8, 
-            0
-        ],  # "f_egored_degree_in" / "f_degree_in"
+        "f_egored_degree_in_rel": ["div", 8, 0],  # "f_egored_degree_in" / "f_degree_in"
         "f_egored_degree_out_rel": [
             "div",
             9,
@@ -531,11 +524,7 @@ def compute_derivated_egonet_feature(
             15,
             7,
         ],  # "f_egored_nr_edges" / "f_ego_nr_edges"
-        "f_ego_edge_density": [
-            "div", 
-            7, 
-            6
-        ],  # "f_ego_nr_edges" / "f_ego_nr_nodes"
+        "f_ego_edge_density": ["div", 7, 6],  # "f_ego_nr_edges" / "f_ego_nr_nodes"
         "f_egored_edge_density": [
             "div",
             15,
@@ -550,13 +539,15 @@ def compute_derivated_egonet_feature(
 
     # compute the derivated feature
     if op[0] == "div":  # division of two base features
-        mask = (ego_base_features[:, op[2]] != 0)
+        mask = ego_base_features[:, op[2]] != 0
         if np.any(mask):
             node_features[mask, col] = (
                 ego_base_features[mask, op[1]] / ego_base_features[mask, op[2]]
             )
     elif op[0] == "div_rel":  # division of two relative features
-        mask = np.logical_and(ego_base_features[:, op[2]] != 0, ego_base_features[:, op[4]] != 0)
+        mask = np.logical_and(
+            ego_base_features[:, op[2]] != 0, ego_base_features[:, op[4]] != 0
+        )
         mask = np.logical_and(mask, ego_base_features[:, op[3]] != 0)
         if np.any(mask):
             node_features[mask, col] = (
@@ -718,7 +709,7 @@ def graph_to_rwalk_features(
     -------
     node_features : pandas dataframe (if input was graph or dataframe) or matrix (if input was matrix)
         Contains the following columns (in this order):
-        
+
         * `'f_rwalk_start_amount'`: average amount on the first leg of the random walk (starting from the node)
         * `'f_rwalk_transfer_out'`: average amount going through the whole walk (the minimum over all legs)
         * `'f_rwalk_out_back'`: average amount coming back to the node
@@ -847,6 +838,7 @@ def graph_to_rwalk_features(
 
 # ---------------------------------------------------------------------------------
 
+
 def build_rwalk_features(feature_list, rwalk_base_features):
     """Build node random walk feature dataframe or matrix given the list of features
        and the base egonet features dataframe or matrix.
