@@ -12,7 +12,7 @@ class Transactions2Graph(BaseEstimator, TransformerMixin):
     * source id (int)
     * destination id (int)
     * amount of money transferred from source to destination
-    
+
     transform it into a graph by cumulating the amounts between two ids and
     counting the associated number of transactions.
 
@@ -31,7 +31,7 @@ class Transactions2Graph(BaseEstimator, TransformerMixin):
     The following code transforms the list into a graph.
 
     .. code-block:: python
-    
+
         from graphomaly.preprocessing.transactions_to_graph import Transactions2Graph
         it2g = Transactions2Graph()
         G = it2g.fit_transform(L)
@@ -79,7 +79,9 @@ class Transactions2Graph(BaseEstimator, TransformerMixin):
             self.column_names = column_names
 
         self.summable_attributes = summable_attributes
-        if self.summable_attributes is not None:   # ignore names if there are no summable attributes
+        if (
+            self.summable_attributes is not None
+        ):  # ignore names if there are no summable attributes
             self.summable_attributes_out_names = summable_attributes_out_names
 
         return self
@@ -100,12 +102,12 @@ class Transactions2Graph(BaseEstimator, TransformerMixin):
         graph : DataFrame or matrix
             Transaction list as a graph. It has the same datatype as X.
             If dataframe, it has the following columns with fixed names:
-    
+
             * `'id_source'`: source node number
             * `'id_destination'`: destination node number
             * `'cumulated_amount'`: the cumulated amount of transactions from source to destination node
             * `'n_transactions'`: number of transactions between the two nodes
-    
+
             Other attributes are built as sums over transactions, as indicated by summable_attributes.
             If matrix, its columns have the meanings above, in exactly this order.
         """
@@ -153,7 +155,7 @@ class Transactions2Graph(BaseEstimator, TransformerMixin):
         if self.summable_attributes is not None:
             graph[self.summable_attributes_out_names] = np.array(
                 group_id[summable_attributes_str].sum()
-        )
+            )
 
         self.feature_names_in_ = [
             "id_source",
@@ -163,7 +165,7 @@ class Transactions2Graph(BaseEstimator, TransformerMixin):
         ]
         if self.summable_attributes is not None:
             self.feature_names_in_ += self.summable_attributes_out_names
-            
+
         self.n_output_features_ = graph.shape[1]
 
         if matrix_io:  # convert back to matrix
